package hclcollector;

#--------------------------------------------------------------------------------------
#  Project	 : hcl
#  File	        : hclcollector
#  Author        : Keerthan Muthurasa <kmuthurasa at mandriva dot com>
#  Created On    : Mon Aug 27 11:15:03 2007
#  Purpose	: Collect hardaware configuration data in order to improve  the 
#  		  compatibility of your system.Data are collected using dmidecode
#  		  tools and lshw tools.Once collected , data are formated in a xml
#  		  format.
# -------------------------------------------------------------------------------------


use File::Temp qw( tempdir );
use File::Basename;
use strict;
use lib qw(/usr/lib/libDrakX);
use common;
use MDK::Common;

require_root_capability();
sub init {	
	my ($dir)=@_;
	my $base=basename($dir);
	rm_rf(glob_("$dir/*"));
	create_lspcidrake_file($dir);
	lshw_collector($dir);
	dmidecode_collector($dir);
	cpuinfo($dir);
	distribution($dir);
	kernel($dir);
	monitor($dir);
	controller($dir);
	input($dir);
	bridge($dir);
	rm_rf("$dir/lspcidrake.out");
	md5file($dir);
	system("cd $dir;tar cfj $base.tar.bz2 *");
}
my $flag=0;
my @pile;
my @indent;


sub dmidecode_collector {
	my ($path)=@_;
	my $start_tag=0;
	my $last_tag;
	my $last_file;
	my $startflag=0;
	my $flagcpt=0;
	my $endflag=0;

my $pid = open(my $PROC, "-|");
if ($pid) {
	foreach (<$PROC>) {
			if (/^$/ || /^Handle/) {
				$start_tag=0;
				if ($last_file ne "" && $last_tag ne "") {
					while (scalar(@pile) > 0) {
						append_to_file($last_file, pop(@indent) . "</" . pop(@pile) . ">\n");
					}
				}
				$last_tag="";
				$last_file="";
			}

			if (/^BIOS Information/) {
				$start_tag=1;
				$last_tag="bios";
				$last_file="$path/bios.xml"; 
				output($last_file, "<-- Information about the bios -->\n") if !-e $last_file;
			}
			if (/^System Information/) {
				$start_tag=1;
				$last_tag="system";
				$last_file="$path/system.xml";
				output($last_file, "<-- Information about the system -->\n") if !-e $last_file;
			}
							
			if (/^Base Board Information/) {
				$start_tag=1;
				$last_tag="base board";
				$last_file="$path/base_board.xml";
				output($last_file, "<-- Information about the base board -->\n") if !-e $last_file;
						}

			if (/^Chassis Information/) {
				$start_tag=1;
				$last_tag="chassis";
				$last_file="$path/chassis.xml";
				output($last_file, "<-- Information about the chassis -->\n") if !-e $last_file;
			}

			if (/^Processor Information/) {
				$start_tag=1;
				$last_tag="processor";
				$last_file="$path/processor.xml";
					}

			if (/^Cache Information/) {
				$start_tag=1;
				$last_tag="cache";
				$last_file="$path/cache.xml";
				output($last_file, "<-- Information about the cache -->\n") if !-e $last_file;
			}

			if (/^Memory Module Information/) {
				$start_tag=1;
				$last_tag="memory module";
				$last_file="$path/memory_module.xml";
				output($last_file, "<-- Information about the memory module -->\n") if !-e $last_file;
			}

			if (/^Port Connector Information/) {
				$start_tag=1;
				$last_tag="port";
				$last_file="$path/port.xml";
				output($last_file, "<-- Information about the ports -->\n") if !-e $last_file;
			}

			if (/^System Slot Information/) {
				$start_tag=1;
				$last_tag="slot";
				$last_file="$path/slot.xml";
				output($last_file, "<-- Information about the slots -->\n") if !-e $last_file;
			}

			if (/^On Board Device Information/) {
				$start_tag=1;
				$last_tag="on board device";
				$last_file="$path/on_board_device.xml";
				output($last_file, "<-- Information about the on board devices -->\n") if !-e $last_file;
			}

			if (/^Memory Device/) {
			 	$start_tag=1;
				$last_tag="memory device";
				$last_file="$path/memory_device.xml";
				output($last_file, "<-- Information about the memor devices -->\n") if !-e $last_file;
			}

			if (($last_tag eq "bios" || $last_tag eq "slot") && $start_tag == 1) { dmidecode("characteristic", $last_file, $_) }
			if ($last_tag eq "processor" && $start_tag == 1) {
				if (/^.*Flags:/) { $startflag=1;$flagcpt=0 }
				$flag = 0 if /^\t/;
				if (/^\t\t/ && $startflag) { $flag=1;$flagcpt++ }
				if ($flagcpt > 0 && $flag == 0) { $endflag=1;$flagcpt=0 }
				else { $endflag = 0 }
				processor($flag, $endflag, $last_file, $_);
	}

			if ($last_tag eq "cache" && $start_tag == 1) { dmidecode("supported_sram_type", $last_file, $_) }
			if (($last_tag eq "system" || $last_tag eq "base board" || $last_tag eq "chassis" || $last_tag eq "memory module" || $last_tag eq "port" || $last_tag eq "on board device" || $last_tag eq "memory device") && $start_tag == 1) { dmidecode("", $last_file, $_) }

				}
}
else {

		exec("dmidecode");
		exit 1;
	}
			}
				
sub lshw_collector {
	my ($path)=@_;
	my $start_tag=0;
	my $last_file;
	my $last_tag;

	my $pid=open(my $PROC, "-|");
	if ($pid) {
		foreach (<$PROC>) {
			
			if (/^.*\*-/ && $last_tag ne "" && $last_file ne "" && $start_tag == 1) {
				if ($last_tag ne "processor") {
					append_to_file($last_file, "</$last_tag>\n");
				}
				$last_tag="";
				$last_file="";
				$start_tag=0;
	}

			if (/^.*\*-cpu/) {	
				$start_tag=1;
				$last_tag="processor";
				$last_file="$path/processor.xml";
				output($last_file, "<-- Information about processor -->\n") if !-e $last_file;
				append_to_file($last_file, "<processor>\n");
	}	

			if (/^.*\*-cdrom/) {	
				$start_tag=1;
				$last_tag="removable";
				$last_file="$path/removable.xml";
				output($last_file, "<-- Information about removable devices -->\n") if !-e $last_file;
				append_to_file($last_file, "<removable>\n");
		}

			if (/^.*\*-disk/) {
				$start_tag=1;
				$last_tag="storage";
				$last_file="$path/storage.xml";
				output($last_file, "<-- Information about storage devices -->\n") if !-e $last_file;
				append_to_file($last_file, "<storage>\n");
	}

			if (/^.*\*-network/) {
				$start_tag=1;
				$last_tag="network";
				$last_file="$path/network.xml";
				output($last_file, "<-- Information about network devices -->\n") if !-e $last_file;
				append_to_file($last_file, "<network>\n");
}

			if (/^.*\*-display/) {
				$start_tag=1;
				$last_tag="video_card";
				$last_file="$path/video_card.xml";
				output($last_file, "<-- Information about video_cards -->\n") if !-e $last_file;
				append_to_file($last_file, "<video_card>\n");
			}

			if (/^.*\*-communication/) {
				$start_tag=1;
				$last_tag="modem";
				$last_file="$path/modem.xml";
				output($last_file, "<-- Information about communication devices -->\n") if !-e $last_file;
				append_to_file($last_file, "<modem>\n");
			}

			if (/^.*\*-multimedia/) {
				$start_tag=1;
				$last_tag="sound_card";
				$last_file="$path/sound_card.xml";
				output($last_file, "<-- Information about multimedia devices -->\n") if !-e $last_file;
				append_to_file($last_file, "<sound_card>\n");
			}

			if ($last_tag eq "processor" && $start_tag == 1) { lshw_processor($last_file, $_) }
			if ($last_tag eq "removable" && $start_tag == 1) { removable($path, $last_file, $_) }
			if ($last_tag eq "storage" && $start_tag == 1) { storage($path, $last_file, $_) }
			if ($last_tag eq "network" && $start_tag == 1) { network($path, $last_file, $_) }
			if ($last_tag eq "video_card" && $start_tag == 1) { video_card($path, $last_file, $_) }
			if ($last_tag eq "modem" && $start_tag == 1) { modem($path, $last_file, $_) }
			if ($last_tag eq "sound_card" && $start_tag == 1) { sound_card($path, $last_file, $_) }
						}
	}
	else {
		exec("lshw");
		exit 1;
	}
}
							
sub cpuinfo {
    my ($path)=@_;
    my $procCount = 0;
    foreach (cat_('/proc/cpuinfo')) {
        $procCount++ if /^processor/;
    }
    append_to_file("$path/processor.xml", qq(<processor_count>$procCount</processor_count>
</processor>
));
						}

sub dmidecode {
	my ($subtag, $file, $line)=@_;
	if ($line !~ m/^\t/) {
		$line =~ s/ /_/g;
		$line =~ s/\n$//g;
		$line = trim($line);
		$line = lc($line);
		append_to_file($file, "<$line>\n");
		push(@pile, $line);
		push(@indent, "");
	}

	if ($line =~ m/^\t\t/) {
		$line =~ s/\n//g;
		$line = trim($line);	
		append_to_file($file, "\t\t<$subtag>$line<\\$subtag>\n");
					}
	if ($line =~ m/^\t/)
	{
		if ($indent[scalar(@indent)-1] eq "\t")
		{
			append_to_file($file, pop(@indent) . "<\\" . pop(@pile) . ">\n");
				}
	
		$line = trim($line);
		$line =~ s/\n$//g;
		my @split = split(':', $line);
		$split[0] =~ s/ /_/g;
		$split[0] = lc($split[0]);
		$split[1] = trim($split[1]);
		if ($split[1] ne "")
		{
			append_to_file($file, "\t<$split[0]>$split[1]<\\$split[0]>\n");
				}
				else {
			append_to_file($file, "\t<$split[0]>\n");
			push(@pile, $split[0]);
			push(@indent, "\t");
				}
}


}

sub distribution {
	my ($path)=@_;
	my $file="$path/distribution.xml";
	
	foreach (cat_("/etc/mandriva-release")) {
		s/for/:/;
		my @split=split(":", $_);
		$split[0]=trim($split[0]);
		$split[1]=trim($split[1]);
		output($file, "<-- Information about linux distribution-->\n");
		append_to_file($file, "<linux_distribution>\n");
		append_to_file($file, "\t<distribution>$split[0]</distribution>\n");
		append_to_file($file, "\t<architecture>$split[1]</architecture>\n");
		append_to_file($file, "</linux_distribution>\n");
	}
	}	

sub monitor {
	my ($path)=@_;
	my $start=0;
	my $file="$path/monitor.xml";

	foreach (cat_("/etc/X11/xorg.conf")) {
		$_=trim($_);
		if (/^Section "Monitor"/i) { $start=1;output($file, "<-- Information about monitor-->\n<monitor>\n") }
	if ($start == 1) {
			if (!/^Section "Monitor"/i) {
				my @split = split(" ", $_);
				$split[1] =~ s/"//;
				append_to_file($file, "\t<description>$split[1]</description>\n") if /^.*Identifier/i;
				append_to_file($file, "\t<vendor>$split[1]</vendor>\n") if /^.*VendorName/i;
				append_to_file($file, "\t<model>$split[1]</vendor>\n") if /^.*ModelName/i;
				append_to_file($file, "\t<horizontal_sync>$split[1]</horizontal_sync>\n") if /^.*HorizSync/i;
				append_to_file($file, "\t<vertical_refresh>$split[1]</vertical_refresh>\n") if /^.*VertRefresh/i;
		}
		}
		if (/^.*EndSection/i && $start == 1) { $start=0;append_to_file($file, "</monitor>\n") }
	}
}

sub kernel {
	my ($path)=@_;
	my $file="$path/kernel.xml";
	output($file, "<-- Information about kernel -->\n");
	append_to_file($file, "<linux_kernel>\n");
	append_to_file($file, "\t<kernel>");
	system("uname -r >> $file");
	append_to_file($file, "\t</kernel>\n");
	append_to_file($file, "</linux_kernel>\n");
}

sub lshw_processor {
	my ($file, $line) = @_;
			my @data = split(': ', $line);
			$data[0] = trim($data[0]);
			$data[0] =~ s/ /_/;
			$data[1] = trim($data[1]);
			$data[1] =~ s/\n//;

		if ($line !~ /^.*\*-cpu/ && $line !~ /^.*capabilities:/ && $line !~ /^.*description:/ && $line !~ /^.*physical id:/ && $line !~ /^.*bus info:/ && $line !~ /^.*slot:/ && $line !~ /^.*width:/ && $line !~ /^.*size:/) {
			append_to_file($file, "<$data[0]>$data[1]</$data[0]>\n");
	}
}


sub md5file {
	require  Digest::MD5;
	my ($path)=@_;
	output("$path/checksum", "");
	foreach my $file (glob_("$path/*")) {
		if ($file !~ /checksum/) {
			open(my $FILE,$file);
			binmode($FILE);
        		my $sum = Digest::MD5->new->addfile(*$FILE)->hexdigest;
			append_to_file("$path/checksum", "$sum  " . basename($file) . "\n");
	}
}
}

sub create_lspcidrake_file {
	my ($path)=@_;
	system("lspcidrake -v >$path/lspcidrake.out");
}

sub trim($) {
	my $string = shift;
	$string =~ s/^\s+//;
	$string =~ s/\s+$//;

	#substitution of special characters
	
	$string =~ s/\[//;
	$string =~ s/\]//;
	$string =~ s/\{//;
	$string =~ s/\}//;
	$string =~ s/\(//;
	$string =~ s/\)//;
	#$string =~ s/(\*|\+|\-|\.|\^|\$|\?|\\|\||"|'//;
	return $string;

}

sub processor {
	my ($flag, $eflag, $file, $line) = @_;
			my @data = split(': ', $line);
			$data[0] = trim($data[0]);
			$data[1] = trim($data[1]);
			$data[1] =~ s/\n//;

	append_to_file($file, "<family>$data[1]</family>\n") if $line =~ /^.*Family:/;
	append_to_file($file, "<flags>\n") if $line =~ /^.*Flags:/;

	if ($flag) {
		$line = trim($line);
		$line =~ s/\n$//;
		append_to_file($file, "\t<flag>$line</flag>\n");
	}
	append_to_file($file, "</flags>\n") if $eflag;
	append_to_file($file, "<socket>$data[1]</socket>\n") if $line =~ /^.*Upgrade:/;
}

sub removable  {
	my ($path, $file, $line) = @_;
			my @data = split(': ', $line);
			$data[0] = trim($data[0]);
			$data[0] =~ s/ /_/;
			$data[1] = trim($data[1]);
			$data[1] =~ s/\n//;

		if (/^.*product:/) {
			my $product=$data[1];
			$product = trim($product);
			foreach my $info (cat_("$path/lspcidrake.out")) {
				$info = trim($info);
				if ($product ne "" && $info =~ /^.*\|$product/) {
					$info =~ s/^.*vendor:/vendor:/;
					$info =~ s/\)//;
					my @ids = split(" ",$info);
					for (my $cpt=0; $cpt < scalar(@ids); $cpt++) {
						my @detail = split(":", $ids[$cpt]);
						$detail[0] = trim($detail[0]);
						$detail[1] = trim($detail[1]);
						append_to_file($file, "<$detail[0]_id>$detail[1]</$detail[0]_id>\n");
					}
						$product="";
				}
			}
		}

		if ($line !~ /^.*\*-cdrom/ && $line !~ /^.*physical id:/ && $line !~ /^.*logical name:/) {
				if ($line =~ /^.*capabilities:/) {
					my @split = split(" ", $data[1]);
				append_to_file($file, "<$data[0]>\n");
					while (scalar(@split) > 0) {
					append_to_file($file, "\t<capability>" . pop(@split) . "</capability>\n");
					}
				append_to_file($file, "</$data[0]>\n");
				}
				elsif ($line =~ /^.*bus info:/) {
					my @split = split("@", $data[1]);
				append_to_file($file, "<$data[0]>$split[0]</$data[0]>\n");
				}
				else {
				append_to_file($file, "<$data[0]>$data[1]</$data[0]>\n");
		}
	}
}

sub storage  {
	my ($path, $file, $line) = @_;
			my @data = split(': ', $line);
			$data[0] = trim($data[0]);
			$data[0] =~ s/ /_/;
			$data[1] = trim($data[1]);
			$data[1] =~ s/\n//;

	if (/^.*product:/) {
		my $product=$data[1];
		$product = trim($product);
		foreach my $info (cat_("$path/lspcidrake.out")) {
			$info = trim($info);
			if ($product ne "" && $info =~ /^.*\|$product/) {
				$info =~ s/^.*vendor:/vendor:/;
				$info =~ s/\)//;
				my @ids = split(" ", $info);
				for (my $cpt=0; $cpt < scalar(@ids); $cpt++) {
					my @detail = split(":", $ids[$cpt]);
					$detail[0] = trim($detail[0]);
					$detail[1] = trim($detail[1]);
					append_to_file($file, "<$detail[0]_id>$detail[1]</$detail[0]_id>\n");
				}
					$product="";
			}
		}
	}

	if ($line !~ /^.*\*-disk/ && $line !~ /^.*physical id:/ && $line !~ /^.*logical name:/) {
				if ($line =~ /^.*capabilities:/) {
					my @split = split(" ", $data[1]);
			append_to_file($file, "<$data[0]>\n");
					while (scalar(@split) > 0) {
					append_to_file($file, "\t<capability>" . pop(@split) . "</capability>\n");
					}
			append_to_file($file, "</$data[0]>\n");
				}
				elsif ($line =~ /^.*bus info:/) {
					my @split = split("@", $data[1]);
			append_to_file($file, "<$data[0]>$split[0]</$data[0]>\n");
				}
				else {
			append_to_file($file, "<$data[0]>$data[1]</$data[0]>\n");
		}

	}
}

sub controller  {
	my ($path) = @_;
	my $file="$path/controller.xml";
	my $start=0;

	output($file, "<-- Information about controllers -->\n");
	my $pid=open(my $PROC, "-|");
	if ($pid) {
			foreach (<$PROC>) {
	if ($start == 1) {
					my @data = split(': ', $_);
			$data[0] = trim($data[0]);
			$data[0] =~ s/ /_/;
			$data[1] = trim($data[1]);
			$data[1] =~ s/\n//;

					if (/^.*product:/) {
						my $product=$data[1];
						$product = trim($product);
						foreach my $info (cat_("$path/lspcidrake.out")) {
							$info = trim($info);
							if ($product ne "" && $info =~ /^.*\|$product/) {
								$info =~ s/^.*vendor:/vendor:/;
								$info =~ s/\)//;
								my @ids = split(" ", $info);
								for (my $cpt=0; $cpt < scalar(@ids); $cpt++) {
									my @detail = split(":", $ids[$cpt]);
									$detail[0] = trim($detail[0]);
									$detail[1] = trim($detail[1]);
									append_to_file($file, "<$detail[0]_id>$detail[1]</$detail[0]_id>\n");
								}
									$product="";
							}
						}
					}
					
					if (!/^.*physical id:/ && !/^.*logical name:/ && !/^.*\*-/) {
						if (/^.*capabilities:/ || /^.*configuration:/) {
							if (/^.*capabilities:/) {
						my @split = split(" ", $data[1]);
								append_to_file($file, "<$data[0]>\n");
						while (scalar(@split) > 0) {
									append_to_file($file, "\t<capability>" . pop(@split) . "</capability>\n");
						}
								append_to_file($file, "</$data[0]>\n");
					}
					else {
						my @split = split(" ", $data[1]);
						my $find = 0;
						while (scalar(@split) > 0 && $find == 0) {
							my $tmp = pop(@split);
							if ($tmp =~ /module/) {
								my @split_ = split("=", $tmp);
										append_to_file($file, "<module>$split_[1]</module>\n");
								$find = 1;
							}
						}

					}

				}
						elsif (/^.*bus info:/) {
					my @split = split("@", $data[1]);
							append_to_file($file, "<$data[0]>$split[0]</$data[0]>\n");
				}
				else {
							append_to_file($file, "<$data[0]>$data[1]</$data[0]>\n");
				}
			}
				
	}

			if (/^.*\*-/ && $start == 1) {
		if ($start == 1) {
					append_to_file($file, "</controller>\n");
		}
       		$start = 0;
	}
			if (/^.*\*-/ && !/^.*\*-core/) {
		$start = 1;
				append_to_file($file, "<controller>\n");
			}
		}
		append_to_file($file, "</controller>\n");	
	}
	else {
		exec("lshw -class bus");
		exit 1;
	}	

}

sub network  {

    my ($path, $file, $line) = @_;
			my @data = split(': ', $line);
			$data[0] = trim($data[0]);
			$data[0] =~ s/ /_/;
			$data[1] = trim($data[1]);
			$data[1] =~ s/\n//;

    if (/^.*product:/) {
        my $product=$data[1];
        $product = trim($product);
        foreach my $info (cat_("$path/lspcidrake.out")) {
            $info = trim($info);
            if ($product ne "" && $info =~ /^.*\|$product/) {
                $info =~ s/^.*vendor:/vendor:/;
                $info =~ s/\)//;
                my @ids = split(" ", $info);
                for (my $cpt=0; $cpt < scalar(@ids); $cpt++) {
                    my @detail = split(":", $ids[$cpt]);
                    $detail[0] = trim($detail[0]);
                    $detail[1] = trim($detail[1]);
                    append_to_file($file, "<$detail[0]_id>$detail[1]</$detail[0]_id>\n");
                }
                $product="";
            }
        }
    }

    if ($line !~ /^.*\*-network/ && $line !~ /^.*physical id:/) {
				if ($line =~ /^.*capabilities:/ || $line =~ /^.*configuration:/) {
					if ($line =~ /^.*capabilities:/) {
						my @split = split(" ", $data[1]);
                append_to_file($file, "<$data[0]>\n");
						while (scalar(@split) > 0) {
				append_to_file($file, "\t<capability>" . pop(@split) . "</capability>\n");
					}
                append_to_file($file, "</$data[0]>\n");
            } else {
						my @split = split(" ", $data[1]);
						while (scalar(@split) > 0) {
							my $tmp = pop(@split);
							my @split_ = split("=", $tmp);
				append_to_file($file, "<$split_[0]>$split_[1]</$split_[0]>\n");
					}
				}
        } elsif ($line =~ /^.*bus info:/) {
					my @split = split("@", $data[1]);
            append_to_file($file, "<$data[0]>$split[0]</$data[0]>\n");
        } else {
            append_to_file($file, "<$data[0]>$data[1]</$data[0]>\n");
		}
	}
}



sub video_card  {
    my ($path, $file, $line) = @_;
			my @data = split(': ', $line);
			$data[0] = trim($data[0]);
			$data[0] =~ s/ /_/;
			$data[1] = trim($data[1]);
			$data[1] =~ s/\n//;

    if (/^.*product:/) {
        my $product=$data[1];
        $product = trim($product);
        foreach my $info (cat_("$path/lspcidrake.out")) {
            $info = trim($info);
            if ($product ne "" && $info =~ /^.*\|$product/) {
                $info =~ s/^.*vendor:/vendor:/;
                $info =~ s/\)//;
                my @ids = split(" ", $info);
                for (my $cpt=0; $cpt < scalar(@ids); $cpt++) {
				my @detail = split(":", $ids[$cpt]);
				$detail[0] = trim($detail[0]);
				$detail[1] = trim($detail[1]);
				append_to_file($file, "<$detail[0]_id>$detail[1]</$detail[0]_id>\n");
                }
                $product="";
            }
        }
    }

    if ($line !~ /^.*physical id:/ && $line !~ /^.*\*-display/) {
				if ($line =~ /^.*capabilities:/ || $line =~ /^.*configuration:/) {
					if ($line =~ /^.*capabilities:/) {
						my @split = split(" ", $data[1]);
                append_to_file($file, "<$data[0]>\n");
						while (scalar(@split) > 0) {
				append_to_file($file, "\t<capability>" . pop(@split) . "</capability>\n");
					}
                append_to_file($file, "</$data[0]>\n");
            } else {
						my @split = split(" ", $data[1]);
						while (scalar(@split) > 0) {
							my $tmp = pop(@split);
							my @split_ = split("=", $tmp);
				append_to_file($file, "<$split_[0]>$split_[1]</$split_[0]>\n");
					}
				}
        } elsif ($line =~ /^.*bus info:/) {
					my @split = split("@", $data[1]);
            append_to_file($file, "<$data[0]>$split[0]</$data[0]>\n");
        } else {
            append_to_file($file, "<$data[0]>$data[1]</$data[0]>\n");
		}
	}
}

sub input  {
    my ($path) = @_;
    my $file="$path/input.xml";
    my $start=0;
    my $pid=open(my $PROC, "-|");
    output($file, "<-- Information about input devices -->\n");
    if ($pid) {
        foreach (<$PROC>) {
	if ($start == 1) {
                my  @data = split(': ', $_);
			$data[0] = trim($data[0]);
			$data[0] =~ s/ /_/;
			$data[1] = trim($data[1]);
			$data[1] =~ s/\n//;

                if (/^.*product:/) {
				my $product=$data[1];
				$product = trim($product);
				foreach my $info (cat_("$path/lspcidrake.out")) {
                        $info = trim($info);
                        if ($product ne "" && $info =~ /^.*\|$product/) {
                            $info =~ s/^.*vendor:/vendor:/;
                            $info =~ s/\)//;
                            my @ids = split(" ", $info);
                            for (my $cpt=0; $cpt < scalar(@ids); $cpt++) {
                                my @detail = split(":", $ids[$cpt]);
                                $detail[0] = trim($detail[0]);
                                $detail[1] = trim($detail[1]);
                                append_to_file($file, "<$detail[0]_id>$detail[1]</$detail[0]_id>\n");
                            }
                            $product="";
                        }
				}
                }
					
                if (!/^.*physical id:/ && !/^.*\*-/) {
				if (/^.*capabilities:/ || /^.*configuration:/) {
                        if (/^.*capabilities:/) {
						my @split = split(" ", $data[1]);
                            append_to_file($file, "<$data[0]>\n");
						while (scalar(@split) > 0) {
                                append_to_file($file, "\t<capability>" . pop(@split) . "</capability>\n");
					}
                            append_to_file($file, "</$data[0]>\n");
                        } else {
						my @split = split(" ", $data[1]);
						my $find = 0;
						while (scalar(@split) > 0 && $find == 0) {
							my $tmp = pop(@split);
							if ($tmp =~ /module/) {
								my @split_ = split("=", $tmp);
                                    append_to_file($file, "<module>$split_[1]</module>\n");
								$find = 1;
							}
						}
				}
				} elsif (/^.*bus info:/) {
					my @split = split("@", $data[1]);
                        append_to_file($file, "<$data[0]>$split[0]</$data[0]>\n");
				} else {
                        append_to_file($file, "<$data[0]>$data[1]</$data[0]>\n");
				}
			}
	}

            if (/^.*\*-/ && $start == 1) {
		if ($start == 1) {
				append_to_file($file, "</input>\n");
		}
       		$start = 0;
	}
            if (/^.*\*-/) {
		$start = 1;
                append_to_file($file, "<input>\n");
	}	
        }
        append_to_file($file, "</input>\n");
    } else {
        exec("lshw -class input");
        exit 1;
	}	

}



sub modem  {
    my ($path, $file, $line) = @_;
			my @data = split(': ', $line);
			$data[0] = trim($data[0]);
			$data[0] =~ s/ /_/;
			$data[1] = trim($data[1]);
			$data[1] =~ s/\n//;

    if (/^.*product:/) {
        my $product=$data[1];
        $product = trim($product);
        foreach my $info (cat_("$path/lspcidrake.out")) {
            $info = trim($info);
            if ($product ne "" && $info =~ /^.*\|$product/) {
                $info =~ s/^.*vendor:/vendor:/;
                $info =~ s/\)//;
                my @ids = split(" ", $info);
                for (my $cpt=0; $cpt < scalar(@ids); $cpt++) {
				my @detail = split(":", $ids[$cpt]);
				$detail[0] = trim($detail[0]);
				$detail[1] = trim($detail[1]);
				append_to_file($file, "<$detail[0]_id>$detail[1]</$detail[0]_id>\n");
                }
                $product="";
            }
        }
    }
					
    if ($line !~ /^.*physical id:/ && $line !~ /^.*\*-/) {
				if ($line =~ /^.*capabilities:/ || $line =~ /^.*configuration:/) {
					if ($line =~ /^.*capabilities:/) {
						my @split = split(" ", $data[1]);
                append_to_file($file, "<$data[0]>\n");
						while (scalar(@split) > 0) {
				append_to_file($file, "\t<capability>" . pop(@split) . "</capability>\n");
					}
                append_to_file($file, "</$data[0]>\n");
            } else {
						my @split = split(" ", $data[1]);
						my $find = 0;
						while (scalar(@split) > 0 && $find == 0) {
							my $tmp = pop(@split);
							if ($tmp =~ /module/) {
								my @split_ = split("=", $tmp);
                        append_to_file($file, "<module>$split_[1]</module>\n");
								$find = 1;
							}
						}

					}

        } elsif ($line =~ /^.*bus info:/) {
					my @split = split("@", $data[1]);
            append_to_file($file, "<$data[0]>$split[0]</$data[0]>\n");
        } else {
            append_to_file($file, "<$data[0]>$data[1]</$data[0]>\n");
	}
	}	
}

sub bridge  {
    my ($path) = @_;
    my $file="$path/bridge.xml";
    my $start=0;
    output($file, "<-- Information about bridges -->\n");
    my $pid=open(my $PROC, "-|");
    if ($pid) {
        foreach (<$PROC>) {

	if ($start == 1) {
                my @data = split(': ', $_);
			$data[0] = trim($data[0]);
			$data[0] =~ s/ /_/;
			$data[1] = trim($data[1]);
			$data[1] =~ s/\n//;

                if (/^.*product:/) {
				my $product=$data[1];
				$product = trim($product);
				foreach my $info (cat_("$path/lspcidrake.out")) {
                        $info = trim($info);
                        if ($product ne "" && $info =~ /^.*\|$product/) {
                            $info =~ s/^.*vendor:/vendor:/;
                            $info =~ s/\)//;
                            my @ids = split(" ",$info);
                            for (my $cpt=0; $cpt < scalar(@ids); $cpt++) {
                                my @detail = split(":", $ids[$cpt]);
                                $detail[0] = trim($detail[0]);
                                $detail[1] = trim($detail[1]);
                                append_to_file($file, "<$detail[0]_id>$detail[1]</$detail[0]_id>\n");
                            }
                            $product="";
                        }
				}
                }
					
                if (!/^.*physical id:/ && !/^.*logical name:/ && !/^.*\*-/) {
				if (/^.*capabilities:/ || /^.*configuration:/) {
                        if (/^.*capabilities:/) {
						my @split = split(" ", $data[1]);
                            append_to_file($file, "<$data[0]>\n");
						while (scalar(@split) > 0) {
                                append_to_file($file, "\t<capability>" . pop(@split) . "</capability>\n");
					}
                            append_to_file($file, "</$data[0]>\n");
                        } else {
						my @split = split(" ", $data[1]);
						my $find = 0;
						while (scalar(@split) > 0 && $find == 0) {
							my $tmp = pop(@split);
							if ($tmp =~ /module/) {
								my @split_ = split("=", $tmp);
                                    append_to_file($file, "<module>$split_[1]</module>\n");
								$find = 1;
							}
						}
				}
				} elsif (/^.*bus info:/) {
					my @split = split("@", $data[1]);
                        append_to_file($file, "<$data[0]>$split[0]</$data[0]>\n");
				} else {
                        append_to_file($file, "<$data[0]>$data[1]</$data[0]>\n");
				}
			}
	}
            if (/^.*\*-/ && $start == 1) {
		if ($start == 1) {
				append_to_file($file, "</bridge>\n");
		}
       		$start = 0;
	}
            if (/^.*\*-/) {
		$start = 1;
                append_to_file($file, "<bridge>\n");
            }
        }
        append_to_file($file, "</bridge>\n");
    } else {
        exec("lshw -class bridge");
        exit 1;
	}	

}

sub sound_card  {

    my ($path, $file, $line) = @_;
			my @data = split(': ', $line);
			$data[0] = trim($data[0]);
			$data[0] =~ s/ /_/;
			$data[1] = trim($data[1]);
			$data[1] =~ s/\n//;

    if (/^.*product:/) {
        my $product=$data[1];
        $product = trim($product);
        foreach my $info (cat_("$path/lspcidrake.out")) {
            $info = trim($info);
            if ($product ne "" && $info =~ /^.*\|$product/) {
                $info =~ s/^.*vendor:/vendor:/;
                $info =~ s/\)//;
                my @ids = split(" ", $info);
                for (my $cpt=0; $cpt < scalar(@ids); $cpt++) {
				my @detail = split(":", $ids[$cpt]);
				$detail[0] = trim($detail[0]);
				$detail[1] = trim($detail[1]);
				append_to_file($file, "<$detail[0]_id>$detail[1]</$detail[0]_id>\n");
                }
                $product="";
            }
        }
    }

    if ($line !~ /^.*multimedia/ && $line !~ /^.*physical id:/) {
				if ($line =~ /^.*capabilities:/ || $line =~ /^.*configuration:/) {
					if ($line =~ /^.*capabilities:/) {
						my @split = split(" ", $data[1]);
                append_to_file($file, "<$data[0]>\n");
						while (scalar(@split) > 0) {
				append_to_file($file, "\t<capability>" . pop(@split) . "</capability>\n");
					}
                append_to_file($file, "</$data[0]>\n");
            } else {
						my @split = split(" ", $data[1]);
						while (scalar(@split) > 0) {
							my $tmp = pop(@split);
							my @split_ = split("=", $tmp);
				append_to_file($file, "<$split_[0]>$split_[1]</$split_[0]>\n");
					}
				}
        } elsif ($line =~ /^.*bus info:/) {
					my @split = split("@", $data[1]);
            append_to_file($file, "<$data[0]>$split[0]</$data[0]>\n");
        } else {
            append_to_file($file, "<$data[0]>$data[1]</$data[0]>\n");
			}
	}

}

1;

